require 'test/lint'
require 'formality'

class ::Form
  include Formality
  attribute :no_default
  attribute :answer, :default => 42
end

lint(Form.new)

tst ".attribute defines reader and writer" do
  assert Form.new.respond_to? :answer
  assert Form.new.respond_to? :answer=
end

tst "an attribute without a :default defaults to nil" do
  assert_equal nil, Form.new.no_default
end

tst ":default option sets a default value" do
  assert_equal 42, Form.new.answer
end

tst "#attribute_names returns an Array of attribute names" do
  assert_equal ["no_default", "answer"], Form.new.attribute_names
end

tst "#attributes returns a HashWithIndifferentAccess" do
  form = Form.new
  assert ActiveSupport::HashWithIndifferentAccess === form.attributes
end

tst "#attributes hash contains all the attribute values" do
  form = Form.new
  assert_equal nil, form.attributes[:no_default]
  assert_equal 42, form.attributes[:answer]
end

tst "#attribute? returns true if the attribute was declared" do
  form = Form.new
  assert form.attribute?(:answer)
  assert form.attribute?(:no_default)
  assert form.attribute?("answer")
  assert form.attribute?("no_default")
end

tst "#assign assigns attribute values" do
  form = Form.new
  form.assign :answer => "hello", :no_default => "some value"

  assert_equal "some value", form.attributes[:no_default]
  assert_equal "hello", form.attributes[:answer]
end

tst "#assign returns the form instance for chaining/assignment convenience" do
  form = Form.new
  the_same = form.assign :answer => "hello", :no_default => "some value"

  assert_equal form, the_same
end


tst "#assign does not assign undeclared attributes" do
  form = Form.new
  def form.undeclared=(*args) raise("shouldn't get here") end
  form.assign(:undeclared => 123,
              :answer => "hello",
              :no_default => "some value")

  assert_equal "some value", form.attributes[:no_default]
  assert_equal "hello", form.attributes[:answer]
end

tst "#assign will assign to @id if an :id key comes in" do
  form = Form.new
  form.assign :id => "123", :answer => "hello", :no_default => "some value"
  assert_equal "123", form.id
end

tst ".assign: just a thin wrapper to not have to call new" do
  form = Form.assign :answer => "hello", :no_default => "some value"

  assert form.kind_of?(Form)
  assert_equal "hello", form.answer
  assert_equal "some value", form.no_default
end

Object.send(:remove_const, :Form)
