#
# Checks `model` for ActiveModel compliance.
#
# The tests are ported directly from ActiveModel::Lint
#

def lint(model)
  tst "responds to :to_key" do
    assert model.respond_to? :to_key
  end

  tst ":to_key returns nil when :persisted? returns false" do
    form = model
    def form.persisted?; false end
    assert_equal nil, form.to_key
  end

  tst "responds to :to_param" do
    assert model.respond_to? :to_param
  end

  tst ":to_param returns nil when :persisted? returns false" do
    form = model
    def form.persisted?; false end
    assert_equal nil, form.to_param
  end

  tst "responds to :to_partial_path" do
    assert model.respond_to? :to_partial_path
  end

  tst ":to_partial_path returns a String" do
    assert String === model.to_partial_path
  end

  tst "responds to :valid?" do
    assert model.respond_to? :valid?
  end

  tst ":valid? returns a Boolean" do
    valid = model.valid?
    assert valid === true || valid === false
  end

  tst "responds to :persisted?" do
    assert model.respond_to? :persisted?
  end

  tst ":persisted? returns a Boolean" do
    persisted = model.persisted?
    assert persisted === true || persisted === false
  end

  tst "responds to :model_name" do
    assert model.class.respond_to? :model_name
  end

  tst ":model_name returns a string with convenience methods" do
    name = model.class.model_name
    assert String === name
    assert String === name.human
    assert String === name.singular
    assert String === name.plural
  end

  tst "responds to :errors" do
    assert model.respond_to? :errors
  end

  tst "error#[] returns an Array" do
    assert Array === model.errors[:hello]
  end

  tst "errors#full_messages returns an Array" do
    assert Array === model.errors.full_messages
  end
end
