require 'test/lint'
require 'formality'
require 'action_dispatch/routing'

#
# Some tests to make sure that Formality forms work
# well with the Rails view helpers (:form_for, et al.)
#

# A little fake AR-like model
::Person = Struct.new(:id, :first, :last) do
  def attributes; Hash[members.zip(values)] end
end

# Our intrepid hero
class ::PersonForm
  include Formality
  attribute :first
  attribute :last
end

lint(PersonForm.new)

# For testing correct route generation delegation
class Route
  include ActionDispatch::Routing::UrlFor

  def self.url_for(*args)
    new.url_for(*args)
  end

  def person_url(person) "person/#{person.id}" end
  def people_url; "people" end
end

tst "#from_model assigns form's attributes" do
  person = Person.new(1, "Arthur", "Dent")
  form = PersonForm.from_model(person)

  assert_equal "Arthur", form.first
  assert_equal "Dent", form.last
end

tst "#from_model assigns form's id" do
  person = Person.new(1, "Arthur", "Dent")
  form = PersonForm.from_model(person)

  assert_equal 1, form.id
end

tst "just to confirm, :id doesn't show up in #attributes" do
  person = Person.new(1, "Arthur", "Dent")
  form = PersonForm.from_model(person)

  assert !form.attributes.has_key?(:id)
  assert !form.attributes.has_key?("id")
end

tst "#persisted? is true if there's an id present" do
  person = Person.new(1, "Arthur", "Dent")
  form = PersonForm.from_model(person)

  assert form.persisted?
end

tst ".model method lets you set the model for the form" do
  klass = PersonForm.dup
  klass.model :person

  assert_equal "Person", klass.model_name
  assert_equal "people", klass.model_name.route_key
  assert_equal "person", klass.model_name.singular_route_key
end

tst ".model lets rails generate routes for underlying model" do
  klass = PersonForm.dup
  klass.model :person

  dent = Person.new(1, "Arthur", "Dent")
  dent_form = klass.from_model(dent)
  anon_form = klass.new

  assert_equal "person/1", Route.url_for(dent_form)
  assert_equal "people", Route.url_for(anon_form)
end

Object.send(:remove_const, :Person)
Object.send(:remove_const, :PersonForm)
